from soupstars import HttpBaseRecipe, ingredient


class NytimesArticleRecipe(HttpBaseRecipe):
    @ingredient
    def title(self):
        return self.read().find('h1').text.replace("\"", "")

    @ingredient
    def authors(self):
        return self.read().find("p", attrs={"itemprop": "author creator"}).text.replace("\"", "")

    @ingredient
    def published_at(self):
        return self.read().find("time").text.replace("\"", "")



from unittest.mock import patch

import pytest


def test_base_can_load():
    from soupstars import BaseRecipe, HttpBaseRecipe


def test_base_can_initialize():
    from soupstars import BaseRecipe
    recipes = BaseRecipe()


def test_load_can_run(recipe):
    load = recipe._load()


def test_lazy_load(Recipe):
    recipe = Recipe(lazy=True)
    assert not hasattr(recipe, "_cached_load")
    recipe._load()
    assert hasattr(recipe, "_cached_load")
    assert recipe._load() == recipe._cached_load


def test_read_can_run(recipe):
    recipe._read()


@patch('soupstars.bases.BaseRecipe._load')
def test_read_triggers_load(_load, recipe):
    recipe._read()
    assert _load.called


def test_set_ingredients_adds_functions(recipe):
    assert 'spices' in recipe._ingredients
    assert 'not_spices' not in recipe._ingredients
    assert recipe.spices == recipe._ingredients['spices']


def test_serialize_items_can_run(recipe):
    result = dict(recipe._serialize_items())
    assert result['spices'] == recipe.spices()
    assert "not_spices" not in result


def test_get_item_can_run(recipe):
    assert recipe['spices'] == recipe.spices()


def test_get_item_can_run_with_lazy_loading(Recipe):
    recipe = Recipe(lazy=True)
    assert not hasattr(recipe, "_cached_load")
    spices = recipe["spices"]
    assert spices == recipe.spices()
    assert hasattr(recipe, "_cached_load")



from unittest.mock import patch, MagicMock

import pytest

mock_response = MagicMock()
mock_response.content = "<html><h1>Hello world</h1><p>From jim</p></html>"

def test_http_base_can_load():
    from soupstars import HttpBaseRecipe


@patch('requests.get')
def test_http_base_can_initialize(get):
    from soupstars import HttpBaseRecipe
    recipes = HttpBaseRecipe('myurl.com')
    assert get.called


@patch('requests.get', return_value=mock_response)
def test_http_recipe_can_parse(get, HttpRecipe):
    http_recipe = HttpRecipe('myurl.com')
    assert http_recipe['title'] == "Hello world"
    # print(requests.get)



import requests
import bs4
import json


def ingredient(func):
    func.__ingredient__ = True
    return func


class BaseRecipe(object):

    @classmethod
    def _return_args(klass, *args, lazy=False):
        return args


    loader = _return_args
    reader = _return_args
    serializer = lambda x: x


    def __init__(self, *args, **kwargs):
        self._args = args
        self._kwargs = kwargs
        self._lazy = kwargs.get('lazy') == True
        self._set_ingredients()
        if not self._lazy:
            self._load()


    def _load(self):
        if hasattr(self, '_cached_load'):
            return self._cached_load
        else:
            self._cached_load = self.__class__.loader(*self._args, **self._kwargs)
            return self._cached_load


    def _read(self):
        return self.__class__.reader(self._load())


    def _set_ingredients(self):
        if hasattr(self, '_ingredients'):
            return self._ingredients
        else:
            self._ingredients = {}
            for attr_name in dir(self):
                attr = getattr(self, attr_name)
                if hasattr(attr, "__ingredient__"):
                    self._ingredients[attr_name] = attr
            else:
                return self._ingredients


    def _serialize_items(self):
        for key in self._ingredients:
            yield key, self.__class__.serializer(self[key])


    def __getitem__(self, key):
        if not hasattr(self, '_cached_load'):
            self._load()
        return self._ingredients[key].__call__()


    def read(self):
        return self._read()


    def load(self):
        return self._load()


class HttpBaseRecipe(BaseRecipe):
    loader = lambda url: requests.get(url).content
    reader = lambda x: bs4.BeautifulSoup(x, features="html.parser")
    serializer = json.dumps

    def json(self):
        return json.dumps(dict(self._serialize_items()), indent=2)
